@@ -10,6 +10,7 @@ import (
 	"github.com/noborus/ov/oviewer"
 	"github.com/spf13/cobra"
 	"github.com/spf13/viper"
+	"golang.org/x/term"
 )
 
 var (
@@ -68,6 +69,8 @@ It supports various compressed files(gzip, bzip2, zstd, lz4, and xz).
 			return Completion(cmd, args)
 		}
 
+		SetRedirect()
+
 		if execCommand {
 			return ExecCommand(cmd, args)
 		}
@@ -159,6 +162,32 @@ func ExecCommand(cmd *cobra.Command, args []string) error {
 	return nil
 }
 
+// Run is a wrapper for oviewer.Run.
+// Evacuate stdout and stderr during the run.
+func SetRedirect() {
+	// Suppress the output of os.Stdout and os.Stderr
+	// because the screen collapses.
+	if term.IsTerminal(int(os.Stdout.Fd())) {
+		tmpStdout := os.Stdout
+		os.Stdout = nil
+		defer func() {
+			os.Stdout = tmpStdout
+		}()
+	} else {
+		oviewer.STDOUTPIPE = os.Stdout
+	}
+
+	if term.IsTerminal(int(os.Stdout.Fd())) {
+		tmpStderr := os.Stderr
+		os.Stderr = nil
+		defer func() {
+			os.Stderr = tmpStderr
+		}()
+	} else {
+		oviewer.STDERRPIPE = os.Stderr
+	}
+}
+
 func init() {
 	config = oviewer.NewConfig()
 	cobra.OnInitialize(initConfig)
