@@ -3,6 +3,9 @@ package oviewer
 import (
 	"fmt"
 	"log"
+	"os"
+	"os/exec"
+	"runtime"
 	"strconv"
 )
 
@@ -10,28 +13,26 @@ import (
 func (root *Root) toggleWrapMode() {
 	root.Doc.WrapMode = !root.Doc.WrapMode
 	root.Doc.x = 0
-	root.setWrapHeaderLen()
-	root.setMessage(fmt.Sprintf("Set WrapMode %t", root.Doc.WrapMode))
+	root.setMessagef("Set WrapMode %t", root.Doc.WrapMode)
 }
 
 //  toggleColumnMode toggles ColumnMode each time it is called.
 func (root *Root) toggleColumnMode() {
 	root.Doc.ColumnMode = !root.Doc.ColumnMode
-	root.setMessage(fmt.Sprintf("Set ColumnMode %t", root.Doc.ColumnMode))
+	root.setMessagef("Set ColumnMode %t", root.Doc.ColumnMode)
 }
 
 // toggleAlternateRows toggles the AlternateRows each time it is called.
 func (root *Root) toggleAlternateRows() {
-	root.Doc.ClearCache()
 	root.Doc.AlternateRows = !root.Doc.AlternateRows
-	root.setMessage(fmt.Sprintf("Set AlternateRows %t", root.Doc.AlternateRows))
+	root.setMessagef("Set AlternateRows %t", root.Doc.AlternateRows)
 }
 
 // toggleLineNumMode toggles LineNumMode every time it is called.
 func (root *Root) toggleLineNumMode() {
 	root.Doc.LineNumMode = !root.Doc.LineNumMode
 	root.ViewSync()
-	root.setMessage(fmt.Sprintf("Set LineNumMode %t", root.Doc.LineNumMode))
+	root.setMessagef("Set LineNumMode %t", root.Doc.LineNumMode)
 }
 
 // toggleFollowMode toggles follow mode.
@@ -47,20 +48,15 @@ func (root *Root) toggleFollowAll() {
 // setDocument sets the Document.
 func (root *Root) setDocument(m *Document) {
 	root.Doc = m
-	root.Clear()
 	root.ViewSync()
 }
 
-// Help is to switch between Help screen and normal screen.
-func (root *Root) Help() {
+// helpDisplay is to switch between helpDisplay screen and normal screen.
+func (root *Root) helpDisplay() {
 	if root.screenMode == Help {
 		root.toNormal()
 		return
 	}
-	root.toHelp()
-}
-
-func (root *Root) toHelp() {
 	root.setDocument(root.helpDoc)
 	root.screenMode = Help
 }
@@ -71,14 +67,11 @@ func (root *Root) logDisplay() {
 		root.toNormal()
 		return
 	}
-	root.toLogDoc()
-}
-
-func (root *Root) toLogDoc() {
 	root.setDocument(root.logDoc)
 	root.screenMode = LogDoc
 }
 
+// toNormal displays a normal document.
 func (root *Root) toNormal() {
 	root.mu.RLock()
 	defer root.mu.RUnlock()
@@ -87,21 +80,6 @@ func (root *Root) toNormal() {
 	root.screenMode = Docs
 }
 
-// setWrapHeaderLen sets the value in wrapHeaderLen.
-func (root *Root) setWrapHeaderLen() {
-	m := root.Doc
-	root.wrapHeaderLen = 0
-	for y := m.SkipLines; y < m.firstLine(); y++ {
-		lc, err := m.lineToContents(y, root.Doc.TabWidth)
-		if err != nil {
-			log.Println(err, "WrapHeaderLen", y)
-			continue
-		}
-		root.wrapHeaderLen++
-		root.wrapHeaderLen += ((len(lc) - 1) / (root.vWidth - root.startX))
-	}
-}
-
 // goLine will move to the specified line.
 func (root *Root) goLine(input string) {
 	lN, err := strconv.Atoi(input)
@@ -110,12 +88,13 @@ func (root *Root) goLine(input string) {
 		return
 	}
 	lN = root.moveLine(lN - 1)
-	root.setMessage(fmt.Sprintf("Moved to line %d", lN+1))
+	root.setMessagef("Moved to line %d", lN+1)
 }
 
+// goLineNumber moves to the specified line number.
 func (root *Root) goLineNumber(ln int) {
 	ln = root.moveLine(ln - root.Doc.firstLine())
-	root.setMessage(fmt.Sprintf("Moved to line %d", ln+1))
+	root.setMessagef("Moved to line %d", ln+1)
 }
 
 // markNext moves to the next mark.
@@ -151,19 +130,19 @@ func (root *Root) addMark() {
 	c := min(root.Doc.topLN+root.Doc.firstLine(), root.Doc.endNum)
 	root.Doc.marked = removeInt(root.Doc.marked, c)
 	root.Doc.marked = append(root.Doc.marked, c)
-	root.setMessage(fmt.Sprintf("Marked to line %d", c-root.Doc.firstLine()+1))
+	root.setMessagef("Marked to line %d", c-root.Doc.firstLine()+1)
 }
 
 // removeMark removes the current line number from the mark.
 func (root *Root) removeMark() {
 	c := root.Doc.topLN + root.Doc.firstLine()
-	oLen := len(root.Doc.marked)
-	root.Doc.marked = removeInt(root.Doc.marked, c)
-	if oLen == len(root.Doc.marked) {
-		root.setMessage(fmt.Sprintf("Not marked line %d", c-root.Doc.firstLine()+1))
-	} else {
-		root.setMessage(fmt.Sprintf("Remove the mark at line %d", c-root.Doc.firstLine()+1))
+	marked := removeInt(root.Doc.marked, c)
+	if len(root.Doc.marked) == len(marked) {
+		root.setMessagef("Not marked line %d", c-root.Doc.firstLine()+1)
+		return
 	}
+	root.Doc.marked = marked
+	root.setMessagef("Remove the mark at line %d", c-root.Doc.firstLine()+1)
 }
 
 // removeAllMark removes all marks.
@@ -177,11 +156,11 @@ func (root *Root) removeAllMark() {
 func (root *Root) setHeader(input string) {
 	num, err := strconv.Atoi(input)
 	if err != nil {
-		root.setMessage(ErrInvalidNumber.Error())
+		root.setMessagef("Set header: %s", ErrInvalidNumber.Error())
 		return
 	}
 	if num < 0 || num > root.vHight-1 {
-		root.setMessage(ErrOutOfRange.Error())
+		root.setMessagef("Set header %d: %s", num, ErrOutOfRange.Error())
 		return
 	}
 	if root.Doc.Header == num {
@@ -189,8 +168,7 @@ func (root *Root) setHeader(input string) {
 	}
 
 	root.Doc.Header = num
-	root.setMessage(fmt.Sprintf("Set header lines %d", num))
-	root.setWrapHeaderLen()
+	root.setMessagef("Set header lines %d", num)
 	root.Doc.ClearCache()
 }
 
@@ -198,11 +176,11 @@ func (root *Root) setHeader(input string) {
 func (root *Root) setSkipLines(input string) {
 	num, err := strconv.Atoi(input)
 	if err != nil {
-		root.setMessage(ErrInvalidNumber.Error())
+		root.setMessagef("Set skip line: %s", ErrInvalidNumber.Error())
 		return
 	}
 	if num < 0 || num > root.vHight-1 {
-		root.setMessage(ErrOutOfRange.Error())
+		root.setMessagef("Set skip line: %s", ErrOutOfRange.Error())
 		return
 	}
 	if root.Doc.SkipLines == num {
@@ -210,30 +188,38 @@ func (root *Root) setSkipLines(input string) {
 	}
 
 	root.Doc.SkipLines = num
-	root.setMessage(fmt.Sprintf("Set skip lines %d", num))
-	root.setWrapHeaderLen()
+	root.setMessagef("Set skip lines %d", num)
 	root.Doc.ClearCache()
 }
 
+// nextDoc displays the next document.
 func (root *Root) nextDoc() {
 	root.setDocumentNum(root.CurrentDoc + 1)
 	root.input.mode = Normal
+	root.debugMessage("next document")
+	root.debugMessage(fmt.Sprintf("cache %v\n", root.Doc.cache.Metrics.String()))
 }
 
+// previouseDoc displays the previous document.
 func (root *Root) previousDoc() {
 	root.setDocumentNum(root.CurrentDoc - 1)
 	root.input.mode = Normal
+	root.debugMessage("previous document")
+	root.debugMessage(fmt.Sprintf("cache %v\n", root.Doc.cache.Metrics.String()))
 }
 
+// switchDocument displays the document of the specified docNum.
 func (root *Root) switchDocument(docNum int) {
 	root.setDocumentNum(docNum)
-	root.debugMessage(fmt.Sprintf("switch document %s", root.Doc.FileName))
+	root.debugMessage("switch document")
+	root.debugMessage(fmt.Sprintf("cache %v\n", root.Doc.cache.Metrics.String()))
 }
 
+// addDocument adds a document and displays it.
 func (root *Root) addDocument(m *Document) {
 	root.mu.Lock()
 	defer root.mu.Unlock()
-	log.Printf("add: %s", m.FileName)
+	root.setMessagef("add %s", m.FileName)
 	m.general = root.Config.General
 
 	root.DocList = append(root.DocList, m)
@@ -242,6 +228,8 @@ func (root *Root) addDocument(m *Document) {
 	root.setDocument(m)
 }
 
+// closeDocument closes the document.
+// If there is only one document, do nothing.
 func (root *Root) closeDocument() {
 	if root.DocumentLen() == 1 {
 		return
@@ -250,9 +238,7 @@ func (root *Root) closeDocument() {
 	root.mu.Lock()
 	defer root.mu.Unlock()
 
-	m := root.Doc
-	log.Printf("close [%d]%s", root.CurrentDoc, m.FileName)
-
+	root.setMessagef("close [%d]%s", root.CurrentDoc, root.Doc.FileName)
 	root.DocList = append(root.DocList[:root.CurrentDoc], root.DocList[root.CurrentDoc+1:]...)
 	if root.CurrentDoc > 0 {
 		root.CurrentDoc--
@@ -262,6 +248,8 @@ func (root *Root) closeDocument() {
 	root.setDocument(doc)
 }
 
+// setDocumentNum actually specifies docNum to display the document.
+// This function is called internally from next / previous / switch / add.
 func (root *Root) setDocumentNum(docNum int) {
 	root.mu.Lock()
 	defer root.mu.Unlock()
@@ -277,6 +265,39 @@ func (root *Root) setDocumentNum(docNum int) {
 	root.setDocument(m)
 }
 
+// suspend suspends the current screen display and runs the shell.
+// It will return when you exit the shell.
+func (root *Root) suspend() {
+	log.Println("Suspend")
+	if err := root.Screen.Suspend(); err != nil {
+		log.Println(err)
+		return
+	}
+	fmt.Println("suspended ov")
+	shell := os.Getenv("SHELL")
+	if shell == "" {
+		if runtime.GOOS == "windows" {
+			shell = "CMD.EXE"
+		} else {
+			shell = "/bin/sh"
+		}
+	}
+	c := exec.Command(shell, "-l")
+	c.Stdin = os.Stdin
+	c.Stdout = os.Stdout
+	c.Stderr = os.Stderr
+	if err := c.Run(); err != nil {
+		log.Println(err)
+	}
+	fmt.Println("resume ov")
+	if err := root.Screen.Resume(); err != nil {
+		log.Println(err)
+	}
+	log.Println("Resume")
+}
+
+// toggleMouse toggles mouse control.
+// When disabled, the mouse is controlled on the terminal side.
 func (root *Root) toggleMouse() {
 	root.Config.DisableMouse = !root.Config.DisableMouse
 	if root.Config.DisableMouse {
@@ -288,27 +309,28 @@ func (root *Root) toggleMouse() {
 	}
 }
 
+// setViewMode switches to the preset display mode.
+// Set header lines and columMode together.
 func (root *Root) setViewMode(input string) {
 	c, ok := root.Config.Mode[input]
 	if !ok {
 		if input != "general" {
-			root.setMessage(fmt.Sprintf("%s mode not found", input))
+			root.setMessagef("%s mode not found", input)
 			return
 		}
 		c = root.General
 	}
 
 	root.Doc.general = c
-	root.setWrapHeaderLen()
 	root.Doc.ClearCache()
 	root.ViewSync()
-	root.setMessage(fmt.Sprintf("Set mode %s", input))
+	root.setMessagef("Set mode %s", input)
 }
 
 // setDelimiter sets the delimiter string.
 func (root *Root) setDelimiter(input string) {
 	root.Doc.ColumnDelimiter = input
-	root.setMessage(fmt.Sprintf("Set delimiter %s", input))
+	root.setMessagef("Set delimiter %s", input)
 }
 
 // setTabWidth sets the tab width.
@@ -323,7 +345,7 @@ func (root *Root) setTabWidth(input string) {
 	}
 
 	root.Doc.TabWidth = width
-	root.setMessage(fmt.Sprintf("Set tab width %d", width))
+	root.setMessagef("Set tab width %d", width)
 	root.Doc.ClearCache()
 }
 
@@ -359,4 +381,5 @@ func (root *Root) updateEndNum() {
 	root.debugMessage(fmt.Sprintf("Update EndNum:%d", root.Doc.BufEndNum()))
 	root.prepareStartX()
 	root.statusDraw()
+	root.Screen.Sync()
 }
