@@ -13,6 +13,7 @@ import (
 )
 
 // content represents one character on the terminal.
+// content is a value that can be set in SetContent() of tcell.
 type content struct {
 	width int
 	mainc rune
@@ -39,9 +40,17 @@ var DefaultContent = content{
 	style: tcell.StyleDefault,
 }
 
+// EOFContent is "~" only.
+var EOFContent = content{
+	mainc: '~',
+	combc: nil,
+	width: 1,
+	style: tcell.StyleDefault.Foreground(tcell.ColorGray),
+}
+
 // parseString converts a string to lineContents.
 // parseString includes escape sequences and tabs.
-func parseString(line string, tabWidth int) lineContents {
+func parseString(str string, tabWidth int) lineContents {
 	lc := lineContents{}
 	state := ansiText
 	csiParameter := new(bytes.Buffer)
@@ -51,7 +60,7 @@ func parseString(line string, tabWidth int) lineContents {
 	bsFlag := false // backspace(^H) flag
 	var bsContent content
 
-	gr := uniseg.NewGraphemes(line)
+	gr := uniseg.NewGraphemes(str)
 	for gr.Next() {
 		runeValue := gr.Runes()[0]
 		c := DefaultContent
@@ -74,9 +83,9 @@ func parseString(line string, tabWidth int) lineContents {
 			}
 		case ansiSubstring:
 			if runeValue == 0x1b {
-				state = ansiEscape
-				continue
+				state = ansiControlSequence
 			}
+			continue
 		case ansiControlSequence:
 			if runeValue == 'm' {
 				style = csToStyle(style, csiParameter)
@@ -342,24 +351,25 @@ func lookupColor(colorNumber int) string {
 	}[colorNumber]
 }
 
-// strToContents converts a single-line string into a content array.
-func strToContents(str string, tabWidth int) lineContents {
-	lc := parseString(str, tabWidth)
-	return lc
+// StrToContents converts a single-line string into a one line of contents.
+// Parse escape sequences, etc.
+// 1 Content matches the characters displayed on the screen.
+func StrToContents(str string, tabWidth int) lineContents {
+	return parseString(str, tabWidth)
 }
 
-// contentsToStr returns a converted string
-// and byte length and contents length conversion table.
-func contentsToStr(lc lineContents) (string, map[int]int) {
+// ContentsToStr returns a converted string
+// and byte position, as well as the content position conversion table.
+func ContentsToStr(lc lineContents) (string, map[int]int) {
 	var buff bytes.Buffer
-	byteMap := make(map[int]int)
+	posCV := make(map[int]int)
 
 	bn := 0
 	for n, c := range lc {
 		if c.mainc == 0 {
 			continue
 		}
-		byteMap[bn] = n
+		posCV[bn] = n
 		_, err := buff.WriteRune(c.mainc)
 		if err != nil {
 			log.Println(err)
@@ -374,6 +384,6 @@ func contentsToStr(lc lineContents) (string, map[int]int) {
 		}
 	}
 	str := buff.String()
-	byteMap[bn] = len(lc)
-	return str, byteMap
+	posCV[bn] = len(lc)
+	return str, posCV
 }
