@@ -14,6 +14,7 @@ func SetupStyle() {
 }
 
 func Test_parseString(t *testing.T) {
+	t.Parallel()
 	type args struct {
 		line     string
 		tabWidth int
@@ -103,6 +104,78 @@ func Test_parseString(t *testing.T) {
 				{width: 1, style: tcell.StyleDefault.Foreground(tcell.ColorMaroon), mainc: rune('d'), combc: nil},
 			},
 		},
+		{
+			name: "bright color",
+			args: args{
+				line: "\x1B[90mc\x1B[m", tabWidth: 8,
+			},
+			want: lineContents{
+				{width: 1, style: tcell.StyleDefault.Foreground(tcell.ColorGray), mainc: rune('c'), combc: nil},
+			},
+		},
+		{
+			name: "bright color back",
+			args: args{
+				line: "\x1B[100mc\x1B[m", tabWidth: 8,
+			},
+			want: lineContents{
+				{width: 1, style: tcell.StyleDefault.Background(tcell.ColorGray), mainc: rune('c'), combc: nil},
+			},
+		},
+		{
+			name: "216 color",
+			args: args{
+				line: "\x1B[38;5;31mc\x1B[m", tabWidth: 8,
+			},
+			want: lineContents{
+				{width: 1, style: tcell.StyleDefault.Foreground(tcell.NewRGBColor(0, 102, 153)), mainc: rune('c'), combc: nil},
+			},
+		},
+		{
+			name: "256",
+			args: args{
+				line: "\x1b[38;5;1mc\x1b[m", tabWidth: 8,
+			},
+			want: lineContents{
+				{width: 1, style: tcell.StyleDefault.Foreground(tcell.Color(tcell.ColorValid + 1)), mainc: rune('c'), combc: nil},
+			},
+		},
+		{
+			name: "256 both",
+			args: args{
+				line: "\x1b[38;5;1;48;5;2mc\x1b[m", tabWidth: 8,
+			},
+			want: lineContents{
+				{width: 1, style: tcell.StyleDefault.Foreground(tcell.Color(tcell.ColorValid + 1)).Background(tcell.Color(tcell.ColorValid + 2)), mainc: rune('c'), combc: nil},
+			},
+		},
+		{
+			name: "24bitcolor",
+			args: args{
+				line: "\x1b[38;2;250;123;250mc\x1b[m", tabWidth: 8,
+			},
+			want: lineContents{
+				{width: 1, style: tcell.StyleDefault.Foreground(tcell.NewRGBColor(250, 123, 250)), mainc: rune('c'), combc: nil},
+			},
+		},
+		{
+			name: "24bitcolor both",
+			args: args{
+				line: "\x1b[38;2;255;0;0;48;2;0;0;255mc\x1b[m", tabWidth: 8,
+			},
+			want: lineContents{
+				{width: 1, style: tcell.StyleDefault.Foreground(tcell.NewRGBColor(255, 0, 0)).Background(tcell.NewRGBColor(0, 0, 255)), mainc: rune('c'), combc: nil},
+			},
+		},
+		{
+			name: "default color",
+			args: args{
+				line: "\x1B[39md\x1B[m", tabWidth: 8,
+			},
+			want: lineContents{
+				{width: 1, style: tcell.StyleDefault, mainc: rune('d'), combc: nil},
+			},
+		},
 		{
 			name: "bold",
 			args: args{
@@ -115,6 +188,35 @@ func Test_parseString(t *testing.T) {
 				{width: 1, style: tcell.StyleDefault.Bold(true), mainc: rune('d'), combc: nil},
 			},
 		},
+		{
+			name: "reset",
+			args: args{
+				line: "\x1B[31mr\x1B[me", tabWidth: 8,
+			},
+			want: lineContents{
+				{width: 1, style: tcell.StyleDefault.Foreground(tcell.ColorMaroon), mainc: rune('r'), combc: nil},
+				{width: 1, style: tcell.StyleDefault, mainc: rune('e'), combc: nil},
+			},
+		},
+		{
+			name: "reset2",
+			args: args{
+				line: "\x1B[31mr\x1Bce", tabWidth: 8,
+			},
+			want: lineContents{
+				{width: 1, style: tcell.StyleDefault.Foreground(tcell.ColorMaroon), mainc: rune('r'), combc: nil},
+				{width: 1, style: tcell.StyleDefault, mainc: rune('e'), combc: nil},
+			},
+		},
+		{
+			name: "substring",
+			args: args{
+				line: "\x1B]sub\x1Bmt", tabWidth: 8,
+			},
+			want: lineContents{
+				{width: 1, style: tcell.StyleDefault, mainc: rune('t'), combc: nil},
+			},
+		},
 		{
 			name: "testOverstrike",
 			args: args{line: "a\ba", tabWidth: 8},
@@ -173,13 +275,14 @@ func Test_parseString(t *testing.T) {
 		t.Run(tt.name, func(t *testing.T) {
 			got := parseString(tt.args.line, tt.args.tabWidth)
 			if !reflect.DeepEqual(got, tt.want) {
-				t.Errorf("parseString() got = %v, want %v", got, tt.want)
+				t.Errorf("parseString() got = %#v, want %#v", got, tt.want)
 			}
 		})
 	}
 }
 
-func Test_parseString2(t *testing.T) {
+func Test_parseStringCombining(t *testing.T) {
+	t.Parallel()
 	type args struct {
 		line     string
 		tabWidth int
@@ -239,6 +342,7 @@ func Test_parseString2(t *testing.T) {
 }
 
 func Test_lastContent(t *testing.T) {
+	t.Parallel()
 	type args struct {
 		lc lineContents
 	}
@@ -291,6 +395,7 @@ func Test_lastContent(t *testing.T) {
 }
 
 func Test_csToStyle(t *testing.T) {
+	t.Parallel()
 	type args struct {
 		style        tcell.Style
 		csiParameter *bytes.Buffer
@@ -338,6 +443,7 @@ func Test_csToStyle(t *testing.T) {
 }
 
 func Test_strToContents(t *testing.T) {
+	t.Parallel()
 	type args struct {
 		line     string
 		tabWidth int
@@ -357,7 +463,7 @@ func Test_strToContents(t *testing.T) {
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			if got := strToContents(tt.args.line, tt.args.tabWidth); !reflect.DeepEqual(got, tt.want) {
+			if got := StrToContents(tt.args.line, tt.args.tabWidth); !reflect.DeepEqual(got, tt.want) {
 				t.Errorf("strToContent() = %v, want %v", got, tt.want)
 			}
 		})
@@ -365,6 +471,7 @@ func Test_strToContents(t *testing.T) {
 }
 
 func Test_contentsToStr(t *testing.T) {
+	t.Parallel()
 	tests := []struct {
 		name  string
 		str   string
@@ -405,7 +512,7 @@ func Test_contentsToStr(t *testing.T) {
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
 			lc := parseString(tt.str, 8)
-			got1, got2 := contentsToStr(lc)
+			got1, got2 := ContentsToStr(lc)
 			if got1 != tt.want1 {
 				t.Errorf("contentsToStr() = %v, want %v", got1, tt.want1)
 			}
