@@ -8,6 +8,8 @@ import (
 	"github.com/gdamore/tcell/v2"
 )
 
+const statusline = 1
+
 // draw is the main routine that draws the screen.
 func (root *Root) draw() {
 	m := root.Doc
@@ -28,9 +30,7 @@ func (root *Root) draw() {
 		lX = m.topLX
 	}
 
-	if m.topLN < 0 {
-		m.topLN = 0
-	}
+	m.topLN = max(m.topLN, 0)
 
 	// Body
 	lX, lY = root.drawBody(lX, lY)
@@ -46,29 +46,35 @@ func (root *Root) draw() {
 	root.Show()
 }
 
+// drawHeader draws header.
 func (root *Root) drawHeader() int {
 	m := root.Doc
 
+	// lY is a logical line.
 	lY := m.SkipLines
+	// lX is the x position of Contents.
 	lX := 0
+
+	// wrap is the number of wrapped lines.
 	wrap := 0
-	for hy := 0; lY < m.firstLine(); hy++ {
+	// hy is the drawing line.
+	hy := 0
+	for ; lY < m.firstLine(); hy++ {
 		if hy > root.vHight {
 			break
 		}
 
-		lc := root.getLineContents(lY, m.TabWidth)
-
+		lc := m.getContents(lY, m.TabWidth)
 		// column highlight
 		if m.ColumnMode {
-			str, byteMap := contentsToStr(lc)
+			str, posCV := ContentsToStr(lc)
 			start, end := rangePosition(str, m.ColumnDelimiter, m.columnNum)
-			root.columnHighlight(lc, byteMap[start], byteMap[end])
+			root.columnHighlight(lc, posCV[start], posCV[end])
 		}
 
 		// line number mode
 		if m.LineNumMode {
-			lc := strToContents(strings.Repeat(" ", root.startX-1), m.TabWidth)
+			lc := StrToContents(strings.Repeat(" ", root.startX-1), m.TabWidth)
 			root.setContentString(0, hy, lc)
 		}
 
@@ -78,14 +84,14 @@ func (root *Root) drawHeader() int {
 		}
 
 		if m.WrapMode {
-			lX, lY = root.wrapContents(hy, lX, lY, lc)
+			lX, lY = root.drawWrapLine(hy, lX, lY, lc)
 			if lX > 0 {
 				wrap++
 			} else {
 				wrap = 0
 			}
 		} else {
-			lX, lY = root.noWrapContents(hy, m.x, lY, lc)
+			lX, lY = root.drawNoWrapLine(hy, m.x, lY, lc)
 		}
 
 		// header style
@@ -94,7 +100,7 @@ func (root *Root) drawHeader() int {
 			root.Screen.SetContent(x, hy, r, c, applyStyle(style, root.StyleHeader))
 		}
 	}
-
+	root.headerLen = hy
 	return lY
 }
 
@@ -103,33 +109,37 @@ func (root *Root) drawBody(lX int, lY int) (int, int) {
 
 	listX, err := root.leftMostX(m.topLN + root.Doc.firstLine() + lY)
 	if err != nil {
-		log.Println(err, "drawBody", m.topLN+lY)
+		log.Printf("drawBody %d:%s", m.topLN+lY, err)
 	}
 	wrap := numOfSlice(listX, lX)
 
+	markStyleWidth := min(root.vWidth, root.Doc.general.MarkStyleWidth)
+
+	root.Doc.lastContentsNum = -1
+
+	// lc, lineStr, byteMap store the previous value.
+	// Because it may be a continuation from the previous line in wrap mode.
 	lastLY := -1
 	var lc lineContents
 	var lineStr string
-	var byteMap map[int]int
-
-	markStyleWidth := min(root.vWidth, root.Doc.general.MarkStyleWidth)
-
-	for y := root.headerLen(); y < root.vHight-1; y++ {
+	var posCV map[int]int
+	for y := root.headerLen; y < root.vHight-statusline; y++ {
 		if lastLY != lY {
-			lc = root.getLineContents(m.topLN+lY, m.TabWidth)
+			lc = m.getContents(m.topLN+lY, m.TabWidth)
+			lineStr, posCV = m.getContentsStr(m.topLN+lY, lc)
+			lastLY = lY
+
 			root.lineStyle(lc, root.StyleBody)
 			root.lnumber[y] = lineNumber{
 				line: -1,
 				wrap: 0,
 			}
-			lineStr, byteMap = root.getContentsStr(m.topLN+lY, lc)
-			lastLY = lY
 		}
 
 		// column highlight
 		if root.Doc.ColumnMode {
 			start, end := rangePosition(lineStr, m.ColumnDelimiter, m.columnNum)
-			root.columnHighlight(lc, byteMap[start], byteMap[end])
+			root.columnHighlight(lc, posCV[start], posCV[end])
 		}
 
 		// search highlight
@@ -141,13 +151,13 @@ func (root *Root) drawBody(lX int, lY int) (int, int) {
 				poss = searchPosition(root.Config.CaseSensitive, lineStr, root.searchWord)
 			}
 			for _, r := range poss {
-				root.searchHighlight(lc, byteMap[r[0]], byteMap[r[1]])
+				root.searchHighlight(lc, posCV[r[0]], posCV[r[1]])
 			}
 		}
 
 		// line number mode
 		if m.LineNumMode {
-			lc := strToContents(fmt.Sprintf("%*d", root.startX-1, m.topLN+lY-m.firstLine()+1), m.TabWidth)
+			lc := StrToContents(fmt.Sprintf("%*d", root.startX-1, m.topLN+lY-m.firstLine()+1), m.TabWidth)
 			for i := 0; i < len(lc); i++ {
 				lc[i].style = applyStyle(tcell.StyleDefault, root.StyleLineNumber)
 			}
@@ -161,14 +171,14 @@ func (root *Root) drawBody(lX int, lY int) (int, int) {
 
 		var nextY int
 		if m.WrapMode {
-			lX, nextY = root.wrapContents(y, lX, lY, lc)
+			lX, nextY = root.drawWrapLine(y, lX, lY, lc)
 			if lX > 0 {
 				wrap++
 			} else {
 				wrap = 0
 			}
 		} else {
-			lX, nextY = root.noWrapContents(y, m.x, lY, lc)
+			lX, nextY = root.drawNoWrapLine(y, m.x, lY, lc)
 		}
 
 		// alternate style applies from beginning to end of line, not content.
@@ -182,7 +192,7 @@ func (root *Root) drawBody(lX int, lY int) (int, int) {
 		}
 
 		// mark style.
-		if intContains(m.marked, m.topLN+lY) {
+		if containsInt(m.marked, m.topLN+lY) {
 			for x := 0; x < markStyleWidth; x++ {
 				r, c, style, _ := root.GetContent(x, y)
 				root.SetContent(x, y, r, c, applyStyle(style, root.StyleMarkLine))
@@ -195,41 +205,8 @@ func (root *Root) drawBody(lX int, lY int) (int, int) {
 	return lX, lY
 }
 
-func (root *Root) getContentsStr(lN int, lc lineContents) (string, map[int]int) {
-	if root.Doc.lastContentsNum != lN {
-		root.Doc.lastContentsStr, root.Doc.lastContentsMap = contentsToStr(lc)
-		root.Doc.lastContentsNum = lN
-	}
-	return root.Doc.lastContentsStr, root.Doc.lastContentsMap
-}
-
-func (root *Root) getLineContents(lN int, tabWidth int) lineContents {
-	org, err := root.Doc.lineToContents(lN, tabWidth)
-	if err == nil {
-		lc := make(lineContents, len(org))
-		copy(lc, org)
-		return lc
-	}
-
-	// EOF
-	width := root.vWidth - root.startX
-	lc := make(lineContents, width)
-	eof := content{
-		mainc: '~',
-		combc: nil,
-		width: 1,
-		style: tcell.StyleDefault.Foreground(tcell.ColorGray),
-	}
-	lc[0] = eof
-
-	for x := 1; x < width; x++ {
-		lc[x] = DefaultContent
-	}
-	return lc
-}
-
-// wrapContents wraps and draws the contents and returns the next drawing position.
-func (root *Root) wrapContents(y int, lX int, lY int, lc lineContents) (int, int) {
+// drawWrapLine wraps and draws the contents and returns the next drawing position.
+func (root *Root) drawWrapLine(y int, lX int, lY int, lc lineContents) (int, int) {
 	if lX < 0 {
 		log.Printf("Illegal lX:%d", lX)
 		return 0, 0
@@ -254,8 +231,8 @@ func (root *Root) wrapContents(y int, lX int, lY int, lc lineContents) (int, int
 	return lX, lY
 }
 
-// noWrapContents draws contents without wrapping and returns the next drawing position.
-func (root *Root) noWrapContents(y int, lX int, lY int, lc lineContents) (int, int) {
+// drawNoWrapLine draws contents without wrapping and returns the next drawing position.
+func (root *Root) drawNoWrapLine(y int, lX int, lY int, lc lineContents) (int, int) {
 	if lX < root.minStartX {
 		lX = root.minStartX
 	}
@@ -300,15 +277,25 @@ func RangeStyle(lc lineContents, start int, end int, style ovStyle) {
 
 // statusDraw draws a status line.
 func (root *Root) statusDraw() {
-	screen := root.Screen
-	style := tcell.StyleDefault
+	leftContents, cursorPos := root.leftStatus()
+	root.setContentString(0, root.statusPos, leftContents)
 
-	for x := 0; x < root.vWidth; x++ {
-		screen.SetContent(x, root.statusPos, 0, nil, style)
+	rightContents := root.rightStatus()
+	root.setContentString(root.vWidth-len(rightContents), root.statusPos, rightContents)
+
+	root.Screen.ShowCursor(cursorPos, root.statusPos)
+}
+
+func (root *Root) leftStatus() (lineContents, int) {
+	if root.input.mode == Normal {
+		return root.normalLeftStatus()
 	}
+	return root.inputLeftStatus()
+}
 
+func (root *Root) normalLeftStatus() (lineContents, int) {
 	number := ""
-	if root.input.mode == Normal && root.DocumentLen() > 1 {
+	if root.DocumentLen() > 1 && root.screenMode == Docs {
 		number = fmt.Sprintf("[%d]", root.CurrentDoc)
 	}
 	follow := ""
@@ -319,7 +306,18 @@ func (root *Root) statusDraw() {
 		follow = "(Follow All)"
 	}
 	leftStatus := fmt.Sprintf("%s%s%s:%s", number, follow, root.Doc.FileName, root.message)
-	leftContents := strToContents(leftStatus, -1)
+	leftContents := StrToContents(leftStatus, -1)
+	color := tcell.ColorWhite
+	if root.CurrentDoc != 0 {
+		color = tcell.Color((root.CurrentDoc + 8) % 16)
+	}
+	for i := 0; i < len(leftContents); i++ {
+		leftContents[i].style = leftContents[i].style.Foreground(tcell.ColorValid + color).Reverse(true)
+	}
+	return leftContents, len(leftContents)
+}
+
+func (root *Root) inputLeftStatus() (lineContents, int) {
 	input := root.input
 	searchMode := ""
 	if input.mode == Search || input.mode == Backsearch {
@@ -333,32 +331,19 @@ func (root *Root) statusDraw() {
 			searchMode += "(Aa)"
 		}
 	}
-	switch input.mode {
-	case Normal:
-		color := tcell.ColorWhite
-		if root.CurrentDoc != 0 {
-			color = tcell.Color((root.CurrentDoc + 8) % 16)
-		}
-
-		for i := 0; i < len(leftContents); i++ {
-			leftContents[i].style = leftContents[i].style.Foreground(tcell.ColorValid + color).Reverse(true)
-		}
-		root.Screen.ShowCursor(len(leftContents), root.statusPos)
-	default:
-		p := searchMode + input.EventInput.Prompt()
-		leftStatus = p + input.value
-		root.Screen.ShowCursor(len(p)+input.cursorX, root.statusPos)
-		leftContents = strToContents(leftStatus, -1)
-	}
-	root.setContentString(0, root.statusPos, leftContents)
+	p := searchMode + input.EventInput.Prompt()
+	leftStatus := p + input.value
+	leftContents := StrToContents(leftStatus, -1)
+	return leftContents, len(p) + input.cursorX
+}
 
+func (root *Root) rightStatus() lineContents {
 	next := ""
 	if !root.Doc.BufEOF() {
 		next = "..."
 	}
-	rightStatus := fmt.Sprintf("(%d/%d%s)", root.Doc.topLN, root.Doc.BufEndNum(), next)
-	rightContents := strToContents(rightStatus, -1)
-	root.setContentString(root.vWidth-len(rightStatus), root.statusPos, rightContents)
+	str := fmt.Sprintf("(%d/%d%s)", root.Doc.topLN, root.Doc.BufEndNum(), next)
+	return StrToContents(str, -1)
 }
 
 // setContentString is a helper function that draws a string with setContent.
@@ -369,12 +354,3 @@ func (root *Root) setContentString(vx int, vy int, lc lineContents) {
 	}
 	screen.SetContent(vx+len(lc), vy, 0, nil, tcell.StyleDefault.Normal())
 }
-
-func intContains(list []int, e int) bool {
-	for _, n := range list {
-		if e == n {
-			return true
-		}
-	}
-	return false
-}
