@@ -35,6 +35,8 @@ func (root *Root) main(ctx context.Context, quitChan chan<- struct{}) {
 			}
 			close(quitChan)
 			return
+		case *eventAppSuspend:
+			root.suspend()
 		case *eventUpdateEndNum:
 			root.updateEndNum()
 		case *eventDocument:
@@ -48,15 +50,19 @@ func (root *Root) main(ctx context.Context, quitChan chan<- struct{}) {
 		case *eventPaste:
 			root.getClipboard(ctx)
 		case *eventSearch:
-			root.forwardSearch(ctx, root.Doc.topLN+root.Doc.firstLine()+1, root.input.value)
+			searchMatch := root.setSearch(root.input.value, root.CaseSensitive)
+			root.searchMove(ctx, true, root.Doc.topLN+root.Doc.firstLine()+1, searchMatch)
 		case *eventBackSearch:
-			root.backSearch(ctx, root.Doc.topLN+root.Doc.firstLine()-1, root.input.value)
+			searchMatch := root.setSearch(root.input.value, root.CaseSensitive)
+			root.searchMove(ctx, false, root.Doc.topLN+root.Doc.firstLine()-1, searchMatch)
 		case *viewModeInput:
 			root.setViewMode(ev.value)
 		case *searchInput:
-			root.forwardSearch(ctx, root.Doc.topLN+root.Doc.firstLine(), ev.value)
+			searchMatch := root.setSearch(root.input.value, root.CaseSensitive)
+			root.searchMove(ctx, true, root.Doc.topLN+root.Doc.firstLine(), searchMatch)
 		case *backSearchInput:
-			root.backSearch(ctx, root.Doc.topLN+root.Doc.firstLine(), ev.value)
+			searchMatch := root.setSearch(root.input.value, root.CaseSensitive)
+			root.searchMove(ctx, false, root.Doc.topLN+root.Doc.firstLine(), searchMatch)
 		case *gotoInput:
 			root.goLine(ev.value)
 		case *headerInput:
@@ -114,6 +120,24 @@ func (root *Root) Quit() {
 	}
 }
 
+// eventAppSuspend represents a suspend event.
+type eventAppSuspend struct {
+	tcell.EventTime
+}
+
+// Quit executes a quit event.
+func (root *Root) Suspend() {
+	if !root.checkScreen() {
+		return
+	}
+	ev := &eventAppSuspend{}
+	ev.SetEventNow()
+	err := root.Screen.PostEvent(ev)
+	if err != nil {
+		log.Println(err)
+	}
+}
+
 // Cancel follow mode and follow all mode.
 func (root *Root) Cancel() {
 	root.General.FollowAll = false
