@@ -59,9 +59,9 @@ func (root *Root) inputEvent(ctx context.Context, ev *tcell.EventKey) {
 		if root.Config.Incsearch {
 			switch root.input.mode {
 			case Search:
-				root.incSearch(ctx)
+				root.incSearch(ctx, true)
 			case Backsearch:
-				root.incBackSearch(ctx)
+				root.incSearch(ctx, false)
 			}
 		}
 		return
@@ -170,9 +170,11 @@ func (root *Root) inputKeyEvent(ev *tcell.EventKey) bool {
 func (root *Root) inputCaseSensitive() {
 	root.Config.CaseSensitive = !root.Config.CaseSensitive
 }
+
 func (root *Root) inputIncSearch() {
 	root.Config.Incsearch = !root.Config.Incsearch
 }
+
 func (root *Root) inputRegexpSearch() {
 	root.Config.RegexpSearch = !root.Config.RegexpSearch
 }
