@@ -2,7 +2,6 @@ package oviewer
 
 import (
 	"fmt"
-	"io"
 	"strings"
 
 	"code.rocketnine.space/tslocum/cbind"
@@ -13,6 +12,7 @@ const (
 	actionExit           = "exit"
 	actionCancel         = "cancel"
 	actionWriteExit      = "write_exit"
+	actionSuspend        = "suspend"
 	actionSync           = "sync"
 	actionFollow         = "follow_mode"
 	actionFollowAll      = "follow_all"
@@ -64,10 +64,11 @@ func (root *Root) setHandler() map[string]func() {
 		actionExit:           root.Quit,
 		actionCancel:         root.Cancel,
 		actionWriteExit:      root.WriteQuit,
+		actionSuspend:        root.suspend,
 		actionSync:           root.ViewSync,
 		actionFollow:         root.toggleFollowMode,
 		actionFollowAll:      root.toggleFollowAll,
-		actionHelp:           root.Help,
+		actionHelp:           root.helpDisplay,
 		actionLogDoc:         root.logDisplay,
 		actionMoveDown:       root.moveDown,
 		actionMoveUp:         root.moveUp,
@@ -159,6 +160,7 @@ func GetKeyBinds(bind map[string][]string) map[string][]string {
 		actionPreviousDoc:    {"["},
 		actionCloseDoc:       {"ctrl+k"},
 		actionToggleMouse:    {"ctrl+alt+r"},
+		actionSuspend:        {"ctrl+z"},
 
 		inputCaseSensitive: {"alt+c"},
 		inputIncSearch:     {"alt+i"},
@@ -178,22 +180,26 @@ func (root *Root) setKeyBind(keyBind map[string][]string) error {
 
 	actionHandlers := root.setHandler()
 
-	for a, keys := range keyBind {
-		handler := actionHandlers[a]
+	for name, keys := range keyBind {
+		handler := actionHandlers[name]
 		if handler == nil {
-			return fmt.Errorf("%w for [%s] unknown action", ErrFailedKeyBind, a)
+			return fmt.Errorf("%w for [%s] unknown action", ErrFailedKeyBind, name)
 		}
 
-		if strings.HasPrefix(a, "input_") {
-			setHandler(in, handler, a, keys)
+		if strings.HasPrefix(name, "input_") {
+			if err := setHandler(in, name, keys, handler); err != nil {
+				return err
+			}
 			continue
 		}
-		setHandler(c, handler, a, keys)
+		if err := setHandler(c, name, keys, handler); err != nil {
+			return err
+		}
 	}
 	return nil
 }
 
-func setHandler(c *cbind.Configuration, handler func(), name string, keys []string) error {
+func setHandler(c *cbind.Configuration, name string, keys []string, handler func()) error {
 	for _, k := range keys {
 		mod, key, ch, err := cbind.Decode(k)
 		if err != nil {
@@ -223,72 +229,3 @@ func (root *Root) keyCapture(ev *tcell.EventKey) bool {
 	root.keyConfig.Capture(ev)
 	return true
 }
-
-// KeyBindString returns keybind as a string for help.
-func KeyBindString(k KeyBind) string {
-	var b strings.Builder
-	fmt.Fprintf(&b, "\n\tKey binding\n\n")
-	k.writeKeyBind(&b, actionExit, "quit")
-	k.writeKeyBind(&b, actionCancel, "cancel")
-	k.writeKeyBind(&b, actionWriteExit, "output screen and quit")
-	k.writeKeyBind(&b, actionHelp, "display help screen")
-	k.writeKeyBind(&b, actionLogDoc, "display log screen")
-	k.writeKeyBind(&b, actionSync, "screen sync")
-	k.writeKeyBind(&b, actionFollow, "follow mode toggle")
-	k.writeKeyBind(&b, actionFollowAll, "follow all mode toggle")
-	k.writeKeyBind(&b, actionToggleMouse, "enable/disable mouse")
-	k.writeKeyBind(&b, actionCloseDoc, "close current document")
-
-	fmt.Fprintf(&b, "\n\tMoving\n\n")
-	k.writeKeyBind(&b, actionMoveDown, "forward by one line")
-	k.writeKeyBind(&b, actionMoveUp, "backward by one line")
-	k.writeKeyBind(&b, actionMoveTop, "go to begin of line")
-	k.writeKeyBind(&b, actionMoveBottom, "go to end of line")
-	k.writeKeyBind(&b, actionMovePgDn, "forward by page")
-	k.writeKeyBind(&b, actionMovePgUp, "backward by page")
-	k.writeKeyBind(&b, actionMoveHfDn, "forward a half page")
-	k.writeKeyBind(&b, actionMoveHfUp, "backward a half page")
-	k.writeKeyBind(&b, actionMoveLeft, "scroll to left")
-	k.writeKeyBind(&b, actionMoveRight, "scroll to right")
-	k.writeKeyBind(&b, actionMoveHfLeft, "scroll left half screen")
-	k.writeKeyBind(&b, actionMoveHfRight, "scroll right half screen")
-	k.writeKeyBind(&b, actionGoLine, "number of go to line")
-	k.writeKeyBind(&b, actionNextDoc, "next document")
-	k.writeKeyBind(&b, actionPreviousDoc, "previous document")
-
-	fmt.Fprintf(&b, "\n\tMark position\n\n")
-	k.writeKeyBind(&b, actionMark, "mark current position")
-	k.writeKeyBind(&b, actionRemoveMark, "remove mark current position")
-	k.writeKeyBind(&b, actionRemoveAllMark, "remove all mark")
-	k.writeKeyBind(&b, actionMoveMark, "move to next marked position")
-	k.writeKeyBind(&b, actionMovePrevMark, "move to previous marked position")
-
-	fmt.Fprintf(&b, "\n\tSearch\n\n")
-	k.writeKeyBind(&b, actionSearch, "forward search mode")
-	k.writeKeyBind(&b, actionBackSearch, "backward search mode")
-	k.writeKeyBind(&b, actionNextSearch, "repeat forward search")
-	k.writeKeyBind(&b, actionNextBackSearch, "repeat backward search")
-
-	fmt.Fprintf(&b, "\n\tChange display\n\n")
-	k.writeKeyBind(&b, actionWrap, "wrap/nowrap toggle")
-	k.writeKeyBind(&b, actionColumnMode, "column mode toggle")
-	k.writeKeyBind(&b, actionAlternate, "color to alternate rows toggle")
-	k.writeKeyBind(&b, actionLineNumMode, "line number toggle")
-
-	fmt.Fprintf(&b, "\n\tChange Display with Input\n\n")
-	k.writeKeyBind(&b, actionViewMode, "view mode selection")
-	k.writeKeyBind(&b, actionDelimiter, "delimiter string")
-	k.writeKeyBind(&b, actionHeader, "number of header lines")
-	k.writeKeyBind(&b, actionSkipLines, "number of skip lines")
-	k.writeKeyBind(&b, actionTabWidth, "TAB width")
-
-	fmt.Fprintf(&b, "\n\tKey binding when typing\n\n")
-	k.writeKeyBind(&b, inputCaseSensitive, "case-sensitive toggle")
-	k.writeKeyBind(&b, inputRegexpSearch, "regular expression search toggle")
-	k.writeKeyBind(&b, inputIncSearch, "incremental search toggle")
-	return b.String()
-}
-
-func (k KeyBind) writeKeyBind(w io.Writer, action string, detail string) {
-	fmt.Fprintf(w, "  %-26s * %s\n", "["+strings.Join(k[action], "], [")+"]", detail)
-}
