@@ -36,7 +36,7 @@ func (root *Root) moveLine(lN int) int {
 // Move up one screen.
 func (root *Root) movePgUp() {
 	root.resetSelect()
-	root.moveNumUp(root.statusPos - root.headerLen())
+	root.moveNumUp(root.statusPos - root.headerLen)
 }
 
 // Moves down one screen.
@@ -68,13 +68,13 @@ func (root *Root) limitMoveDown(x int, y int) {
 // Moves up half a screen.
 func (root *Root) moveHfUp() {
 	root.resetSelect()
-	root.moveNumUp((root.statusPos - root.headerLen()) / 2)
+	root.moveNumUp((root.statusPos - root.headerLen) / 2)
 }
 
 // Moves down half a screen.
 func (root *Root) moveHfDn() {
 	root.resetSelect()
-	root.moveNumDown((root.statusPos - root.headerLen()) / 2)
+	root.moveNumDown((root.statusPos - root.headerLen) / 2)
 }
 
 // numOfSlice returns what number x is in slice.
@@ -123,7 +123,6 @@ func (root *Root) moveNumDown(moveY int) {
 	x := m.topLX
 	listX, err := root.leftMostX(num)
 	if err != nil {
-		log.Println(err, num)
 		return
 	}
 	n := numOfReverseSlice(listX, x)
@@ -136,7 +135,7 @@ func (root *Root) moveNumDown(moveY int) {
 			}
 			listX, err = root.leftMostX(num)
 			if err != nil {
-				log.Println(err, num)
+				log.Printf("moveNumDown: %d %s", num, err.Error())
 				return
 			}
 			n = 0
@@ -274,25 +273,25 @@ func (root *Root) moveRight() {
 // columnModeX returns the actual x from m.columnNum.
 func (root *Root) columnModeX() int {
 	m := root.Doc
-	// m.Header+10 = Maximum columnMode target.
-	for i := 0; i < m.Header+10; i++ {
-		lc, err := m.lineToContents(m.topLN+m.Header+i, m.TabWidth)
+	// m.firstLine()+10 = Maximum columnMode target.
+	for i := 0; i < m.firstLine()+10; i++ {
+		lc, err := m.contentsLN(m.topLN+m.firstLine()+i, m.TabWidth)
 		if err != nil {
 			continue
 		}
-		lineStr, byteMap := contentsToStr(lc)
+		lineStr, posCV := ContentsToStr(lc)
 		// Skip lines that do not contain a delimiter.
 		if !strings.Contains(lineStr, m.ColumnDelimiter) {
 			continue
 		}
 
 		start, end := rangePosition(lineStr, m.ColumnDelimiter, m.columnNum)
-		if start < 0 || end < 0 {
+		if start < 0 || end < 0 || (start == len(lineStr)) {
 			m.columnNum--
 			start, end = rangePosition(lineStr, m.ColumnDelimiter, m.columnNum)
 		}
-		sx := byteMap[start]
-		ex := byteMap[end] + 10
+		sx := posCV[start]
+		ex := posCV[end] + 10
 		if root.vWidth > ex {
 			return 0
 		}
@@ -342,15 +341,14 @@ func (root *Root) moveHfRight() {
 // bottomLineNum returns the display start line
 // when the last line number as an argument.
 func (root *Root) bottomLineNum(lN int) (int, int) {
-	hight := (root.vHight - root.headerLen()) - 2
-	if lN < root.headerLen() {
+	if lN < root.headerLen {
 		return 0, 0
 	}
 
+	hight := (root.vHight - root.headerLen) - 2
 	if !root.Doc.WrapMode {
-		return 0, lN - (hight + root.headerLen())
+		return 0, lN - (hight + root.headerLen)
 	}
-
 	// WrapMode
 	lX, lN := root.findNumUp(0, lN, hight)
 	return lX, lN - root.Doc.firstLine()
