@@ -46,11 +46,13 @@ type Root struct {
 	OriginPos int
 	// Original string.
 	OriginStr string
-	//
+	// cancelFunc saves the cancel function, which is a time-consuming process.
 	cancelFunc context.CancelFunc
 
+	// searchWord for on-screen highlighting.
 	searchWord string
-	searchReg  *regexp.Regexp
+	// searchReg for on-screen highlighting.
+	searchReg *regexp.Regexp
 
 	// keyConfig contains the binding settings for the key.
 	keyConfig *cbind.Configuration
@@ -89,8 +91,8 @@ type Root struct {
 	// mouseRectangle is a flag for rectangle selection.
 	mouseRectangle bool
 
-	// wrapHeaderLen is the actual header length when wrapped.
-	wrapHeaderLen int
+	// headerLen is the actual header length when wrapped.
+	headerLen int
 
 	// bottomLN is the last line number displayed.
 	bottomLN int
@@ -113,6 +115,7 @@ type lineNumber struct {
 }
 
 // general structure contains the general of the display.
+// general contains values that determine the behavior of each document.
 type general struct {
 	// TabWidth is tab stop num.
 	TabWidth int
@@ -222,6 +225,16 @@ var (
 	OverLineStyle tcell.Style
 )
 
+// ov output destination.
+var (
+	// Redirect to standard output.
+	//     echo "t" | ov> out
+	STDOUTPIPE *os.File
+	// Redirects the error output of ov --exec.
+	//     ov --exec -- command 2> out
+	STDERRPIPE *os.File
+)
+
 // ScreenMode represents the state of the screen.
 type ScreenMode int
 
@@ -255,6 +268,7 @@ var (
 	ErrSignalCatch = errors.New("signal catch")
 )
 
+// This is a function of tcell.NewScreen but can be replaced with mock.
 var tcellNewScreen = tcell.NewScreen
 
 // NewOviewer return the structure of oviewer.
@@ -283,6 +297,12 @@ func NewOviewer(docs ...*Document) (*Root, error) {
 	}
 	root.Screen = screen
 
+	logDoc, err := NewLogDoc()
+	if err != nil {
+		return nil, err
+	}
+	root.logDoc = logDoc
+
 	return root, nil
 }
 
@@ -321,29 +341,45 @@ func NewConfig() Config {
 }
 
 // Open reads the file named of the argument and return the structure of oviewer.
+// If there is no file name, create Root from standard input.
+// If there is only one file name, create Root from that file,
+// but return an error if the open is an error.
+// If there is more than one file name, create Root from multiple files.
 func Open(fileNames ...string) (*Root, error) {
+	var root *Root
+	var err error
 	switch len(fileNames) {
 	case 0:
-		return openSTDIN()
+		root, err = openSTDIN()
+		if err != nil {
+			return nil, err
+		}
 	case 1:
 		m, err := openFile(fileNames[0])
 		if err != nil {
 			return nil, err
 		}
-		return NewOviewer(m)
+		root, err = NewOviewer(m)
+		if err != nil {
+			return nil, err
+		}
 	default:
-		return openFiles(fileNames)
+		root, err = openFiles(fileNames)
+		if err != nil {
+			return nil, err
+		}
 	}
 
+	return root, nil
 }
 
 // NewRoot returns the structure of the oviewer.
+// NewRoot is a simplified version that can be used externally.
 func NewRoot(read io.Reader) (*Root, error) {
 	m, err := NewDocument()
 	if err != nil {
 		return nil, err
 	}
-
 	if err := m.ReadAll(read); err != nil {
 		return nil, err
 	}
@@ -383,11 +419,12 @@ func openFile(fileName string) (*Document, error) {
 }
 
 func openFiles(fileNames []string) (*Root, error) {
+	errors := make([]string, 0)
 	docList := make([]*Document, 0)
 	for _, fileName := range fileNames {
 		m, err := openFile(fileName)
 		if err != nil {
-			log.Println(err)
+			errors = append(errors, fmt.Sprintf("open error: %s", err))
 			continue
 		}
 		docList = append(docList, m)
@@ -396,8 +433,15 @@ func openFiles(fileNames []string) (*Root, error) {
 	if len(docList) == 0 {
 		return nil, fmt.Errorf("%w: %s", ErrMissingFile, fileNames[0])
 	}
+	root, err := NewOviewer(docList...)
+	if err != nil {
+		return nil, err
+	}
 
-	return NewOviewer(docList...)
+	for _, e := range errors {
+		log.Println(e)
+	}
+	return root, err
 }
 
 // SetConfig sets config.
@@ -456,19 +500,12 @@ func (root *Root) setKeyConfig() (map[string][]string, error) {
 	return keyBind, nil
 }
 
-// NewHelp generates a document for help.
-func NewHelp(k KeyBind) (*Document, error) {
-	help, err := NewDocument()
-	if err != nil {
-		return nil, err
+func (root *Root) SetKeyHandler(name string, keys []string, handler func()) error {
+	c := root.keyConfig
+	if err := setHandler(c, name, keys, handler); err != nil {
+		return err
 	}
-	help.FileName = "Help"
-	str := KeyBindString(k)
-	help.lines = append(help.lines, "\t\t\tov help\n")
-	help.lines = append(help.lines, strings.Split(str, "\n")...)
-	help.eof = 1
-	help.endNum = len(help.lines)
-	return help, err
+	return nil
 }
 
 // Run starts the terminal pager.
@@ -482,23 +519,18 @@ func (root *Root) Run() error {
 	defer watcher.Close()
 	root.SetWatcher(watcher)
 
+	// Do not set the key bindings in NewOviewer
+	// because it is done after loading the config.
 	keyBind, err := root.setKeyConfig()
 	if err != nil {
 		return err
 	}
-
 	help, err := NewHelp(keyBind)
 	if err != nil {
 		return err
 	}
 	root.helpDoc = help
 
-	logDoc, err := NewLogDoc()
-	if err != nil {
-		return err
-	}
-	root.logDoc = logDoc
-
 	if !root.Config.DisableMouse {
 		root.Screen.EnableMouse()
 	}
@@ -535,18 +567,28 @@ func (root *Root) Run() error {
 	sigs := make(chan os.Signal, 1)
 	signal.Notify(sigs, os.Interrupt, syscall.SIGTERM, syscall.SIGHUP, syscall.SIGINT)
 
+	sigSuspend := registerSIGTSTP()
+
 	quitChan := make(chan struct{})
 
 	ctx := context.Background()
 	ctx, cancel := context.WithCancel(ctx)
 	defer cancel()
 
-	go root.main(ctx, quitChan)
+	go func() {
+		// Undo screen when goroutine panic.
+		defer func() {
+			root.Close()
+		}()
+		root.main(ctx, quitChan)
+	}()
 
 	for {
 		select {
 		case <-quitChan:
 			return nil
+		case <-sigSuspend:
+			root.Suspend()
 		case sig := <-sigs:
 			return fmt.Errorf("%w [%s]", ErrSignalCatch, sig)
 		}
@@ -568,6 +610,11 @@ func (root *Root) setMessage(msg string) {
 	root.Show()
 }
 
+func (root *Root) setMessagef(format string, a ...interface{}) {
+	msg := fmt.Sprintf(format, a...)
+	root.setMessage(msg)
+}
+
 func (root *Root) debugMessage(msg string) {
 	if !root.Debug {
 		return
@@ -657,8 +704,7 @@ func (root *Root) prepareView() {
 	root.vHight = max(root.vHight, 1)
 
 	root.lnumber = make([]lineNumber, root.vHight+1)
-	root.setWrapHeaderLen()
-	root.statusPos = root.vHight - 1
+	root.statusPos = root.vHight - statusline
 }
 
 // docSmall returns with bool whether the file to display fits on the screen.
@@ -673,9 +719,9 @@ func (root *Root) docSmall() bool {
 	}
 	hight := 0
 	for y := 0; y < m.BufEndNum(); y++ {
-		lc, err := m.lineToContents(y, root.Doc.TabWidth)
+		lc, err := m.contentsLN(y, root.Doc.TabWidth)
 		if err != nil {
-			log.Println(err, y)
+			log.Printf("docSmall %d: %s", y, err)
 			continue
 		}
 		hight += 1 + (len(lc) / root.vWidth)
@@ -695,7 +741,7 @@ func (root *Root) WriteOriginal() {
 			break
 		}
 
-		lc, err := m.lineToContents(n, root.Doc.TabWidth)
+		lc, err := m.contentsLN(n, root.Doc.TabWidth)
 		if err != nil {
 			log.Println(err, n)
 			continue
@@ -723,17 +769,9 @@ func (root *Root) WriteLog() {
 	}
 }
 
-// headerLen returns the actual number of lines in the header.
-func (root *Root) headerLen() int {
-	if root.Doc.WrapMode {
-		return root.wrapHeaderLen
-	}
-	return root.Doc.Header
-}
-
 // leftMostX returns a list of left - most x positions when wrapping.
 func (root *Root) leftMostX(lN int) ([]int, error) {
-	lc, err := root.Doc.lineToContents(lN, root.Doc.TabWidth)
+	lc, err := root.Doc.contentsLN(lN, root.Doc.TabWidth)
 	if err != nil {
 		return nil, err
 	}
@@ -757,39 +795,3 @@ func (root *Root) DocumentLen() int {
 	defer root.mu.RUnlock()
 	return len(root.DocList)
 }
-
-func max(a, b int) int {
-	if a > b {
-		return a
-	}
-	return b
-}
-
-func min(a, b int) int {
-	if a < b {
-		return a
-	}
-	return b
-}
-
-func removeStr(list []string, s string) []string {
-	if len(s) == 0 {
-		return list
-	}
-
-	for n, l := range list {
-		if l == s {
-			list = append(list[:n], list[n+1:]...)
-		}
-	}
-	return list
-}
-
-func removeInt(list []int, c int) []int {
-	for n, l := range list {
-		if l == c {
-			list = append(list[:n], list[n+1:]...)
-		}
-	}
-	return list
-}
