@@ -133,6 +133,9 @@ func (m *Document) ReadFile(fileName string) error {
 		atomic.StoreInt32(&m.changed, 1)
 		close(m.reOpenCh)
 	}()
+	if STDOUTPIPE != nil {
+		reader = io.TeeReader(reader, STDOUTPIPE)
+	}
 
 	return m.ReadAll(reader)
 }
@@ -253,3 +256,13 @@ func (m *Document) append(line string) {
 	m.mu.Unlock()
 	atomic.StoreInt32(&m.changed, 1)
 }
+
+func (m *Document) checkClose() bool {
+	select {
+	case <-m.closeCh:
+		log.Printf("document closed %s", m.FileName)
+		return true
+	default:
+	}
+	return false
+}
